<?php

namespace Mark\Exam\Http\Controllers;

use App\Http\Controllers\PublicController as BaseController;
use Mark\Exam\Interfaces\StudentRepositoryInterface;

class StudentPublicController extends BaseController
{
    // use StudentWorkflow;

    /**
     * Constructor.
     *
     * @param type \Mark\Student\Interfaces\StudentRepositoryInterface $student
     *
     * @return type
     */
    public function __construct(StudentRepositoryInterface $student)
    {
        $this->repository = $student;
        parent::__construct();
    }

    /**
     * Show student's list.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function index()
    {
        $students = $this->repository
        ->pushCriteria(app('Litepie\Repository\Criteria\RequestCriteria'))
        ->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();


        return $this->response->setMetaTitle(trans('$exam::student.names'))
            ->view('exam::student.index')
            ->data(compact('students'))
            ->output();
    }


    /**
     * Show student.
     *
     * @param string $slug
     *
     * @return response
     */
    protected function show($slug)
    {
        $student = $this->repository->scopeQuery(function($query) use ($slug) {
            return $query->orderBy('id','DESC')
                         ->where('slug', $slug);
        })->first(['*']);

        return $this->response->setMetaTitle($$student->name . trans('exam::student.name'))
            ->view('exam::student.show')
            ->data(compact('student'))
            ->output();
    }

}
